unit RzErrDlg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids;

type
  TErrorType = ( etNormal, etWarning, etValidation, etCritical, etSystem, etOther );

  TFrmErrorDlg = class(TForm)
    LstErrors: TListBox;
    BtnProceed: TBitBtn;
    BtnCancel: TBitBtn;
    BtnOK: TBitBtn;
    procedure LstErrorsDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure LstErrorsMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
  private
    { Private declarations }
    FErrorBmp : TBitmap;
    FWarningBmp : TBitmap;
  public
    { Public declarations }
    procedure ClearMessages;
    function NumberOfErrors : Integer;
    procedure SetButtons( Proceed : Boolean );
    procedure AddError( Msg : string; ErrorType : TErrorType );
  end;

var
  FrmErrorDlg: TFrmErrorDlg;

implementation

{$R *.DFM}
{$IFDEF WIN32}
{$R RZERRDLG.R32}
{$ELSE}
{$R RZERRDLG.R16}
{$ENDIF}


procedure TFrmErrorDlg.FormCreate(Sender: TObject);
begin
  FErrorBmp := TBitmap.Create;
  FErrorBmp.Handle := LoadBitmap( HInstance, 'RZ_ERROR' );
  FWarningBmp := TBitmap.Create;
  FWarningBmp.Handle := LoadBitmap( HInstance, 'RZ_WARNING' );
end;


procedure TFrmErrorDlg.FormDestroy(Sender: TObject);
begin
  FErrorBmp.Free;
  FWarningBmp.Free;
end;


procedure TFrmErrorDlg.ClearMessages;
begin
  LstErrors.Clear;
end;

function TFrmErrorDlg.NumberOfErrors : Integer;
begin
  Result := LstErrors.Items.Count;
end;


procedure TFrmErrorDlg.SetButtons( Proceed : Boolean );
begin
  BtnProceed.Visible := Proceed;
  BtnCancel.Visible := Proceed;
  BtnOK.Visible := not Proceed;
end;


procedure TFrmErrorDlg.AddError( Msg : string; ErrorType : TErrorType );
begin
  LstErrors.Items.AddObject( Msg, TObject( ErrorType ) );
end;


procedure TFrmErrorDlg.LstErrorsDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Bmp : TBitmap;
  R, DestRct, SrcRct : TRect;
  TransparentColor : TColor;
  ItemStz : array[ 0..255 ] of Char;
begin
  with LstErrors.Canvas do
  begin
    Bmp := TBitmap.Create;
    try
      FillRect( Rect );   { Clear area for icon and text }

      DestRct := Classes.Rect( 0, 0, 18, 18 );
      SrcRct := DestRct;

      { Don't Forget to Set the Width and Height of Destination Bitmap }
      Bmp.Width := 18;
      Bmp.Height := 18;

      if odSelected in State then
        Bmp.Canvas.Brush.Color := clHighlight
      else
        Bmp.Canvas.Brush.Color := LstErrors.Color;

      if TErrorType( LstErrors.Items.Objects[ Index ] ) = etWarning then
      begin
        TransparentColor := FWarningBmp.Canvas.Pixels[ 0, FWarningBmp.Height - 1 ];
        Bmp.Canvas.BrushCopy( DestRct, FWarningBmp, SrcRct, TransparentColor );
        Draw( Rect.Left + 4, Rect.Top + 2, Bmp );
      end
      else
      begin
        TransparentColor := FWarningBmp.Canvas.Pixels[ 0, FErrorBmp.Height - 1 ];
        Bmp.Canvas.BrushCopy( DestRct, FErrorBmp, SrcRct, TransparentColor );
        Draw( Rect.Left + 4, Rect.Top + 2, Bmp );
      end;

      R := Rect;
      Inc( R.Left, 24 );
      Inc( R.Top, 2 );
      StrPCopy( ItemStz, LstErrors.Items[ Index ] );
      DrawText( Handle, ItemStz, -1, R, dt_WordBreak or dt_ExpandTabs );
    finally
      Bmp.Free;
    end;
  end;

end;


procedure TFrmErrorDlg.LstErrorsMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
var
  R : TRect;
  ItemStz : array[ 0..255 ] of Char;
  H : Integer;
begin
  R := Rect( 0, 0, LstErrors.Width - 24, 2 );
  StrPCopy( ItemStz, LstErrors.Items[ Index ] );
  H := DrawText( LstErrors.Canvas.Handle, ItemStz, -1, R,
                 dt_CalcRect or dt_WordBreak or dt_ExpandTabs );
  if H < 20 then
    H := 20;
  Height := H + 6;
end;


procedure TFrmErrorDlg.FormShow(Sender: TObject);
begin
  LstErrors.SetFocus;
end;


end.
